// Copyright (c) 2003 Robin J Carey. All rights reserved.
//
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. The name of the author, Robin J Carey, may not be used to endorse or
//    promote products derived from this software without specific prior
//    written permission.
// 4. This software may not be used for terrorism, paedophilia or crimes
//    against humanity.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
//

# ifndef  OOO__Terminal_h__OOO
# define  OOO__Terminal_h__OOO

# include  <stdio.h>		// For: perror(3), fileno(3)
# include  <stdlib.h>		// For: exit(3)
# include  <termios.h>		// For: tcgetattr(3), tcsetattr(3),
				//      struct termios, ECHO, TCSANOW

class Terminal {
  private:
    struct termios	original;

  public:
    Terminal (void) {
      if (tcgetattr (fileno (stdin), &original) != 0) {
	perror ("tcgetattr(3)");
	exit (EXIT_FAILURE);
      }
    }

    void		Restore (void) const {
      if (tcsetattr (fileno (stdin), TCSANOW, &original) != 0) {
	perror ("tcsetattr(3)");
	exit (EXIT_FAILURE);
      }
    }

    void		RawInput (void) const {
      struct termios	t;

      if (tcgetattr (fileno (stdin), &t) != 0) {
	perror ("tcgetattr(3)");
	exit (EXIT_FAILURE);
      }
# ifdef ICANON
      t.c_lflag &= ~ICANON;
# endif
# ifdef IEXTEN
      t.c_lflag &= ~IEXTEN;
# endif
# ifdef IXOFF
      t.c_iflag &= ~IXOFF;
# endif
# ifdef IXON
      t.c_iflag &= ~IXON;
# endif
      if (tcsetattr (fileno (stdin), TCSANOW, &t) != 0) {
	perror ("tcsetattr(3)");
	exit (EXIT_FAILURE);
      }
    }

    void		LocalEchoOff (void) const {
      struct termios	t;

      if (tcgetattr (fileno (stdin), &t) != 0) {
	perror ("tcgetattr(3)");
	exit (EXIT_FAILURE);
      }
      t.c_lflag &= ~ECHO;
      if (tcsetattr (fileno (stdin), TCSANOW, &t) != 0) {
	perror ("tcsetattr(3)");
	exit (EXIT_FAILURE);
      }
    }
};

# endif		// !OOO__Terminal_h__OOO
